"""
Module with models for calculating melt Fe\ :sup:`3+`\ /Fe\ :sup:`2+` ratios.
"""

from MagmaPandas.Fe_redox.Fe_redox import FeRedox_QFM
from MagmaPandas.Fe_redox.models import (
    Fe3Fe2_borisov,
    Fe3Fe2_kressCarmichael,
    Fe3Fe2_models,
)
