from MagmaPandas.Fe_redox.models.borisov import Fe3Fe2_borisov
from MagmaPandas.Fe_redox.models.kressCarmichael import Fe3Fe2_kressCarmichael

Fe3Fe2_models = {"borisov": Fe3Fe2_borisov, "kressCarmichael": Fe3Fe2_kressCarmichael}
